# -*- coding: utf-8 -*-
"""
Created on Sun Jun 19 14:33:50 2016
Generate plots of Callisto light curve(s) and calibrate
https://file.scirp.org/Html/3-4500329_48780.htm
@author: STEG
"""

import numpy as np
from matplotlib import pyplot as plt

#------------------------------------------------------------------------------

path = 'LC20230614_ADU_Freienbach.txt'

with open(path) as file:
    line = file.readline()
header = line.split(',')


f = open(path, 'r')
sky = np.genfromtxt(f, delimiter=',',skip_header=1)
f.close()

time = sky[:,0]
channel = 7  # 1...(7)....10

cali = 1./1024 * 2500./25.4 # 1024 digit/FS, 2500 mV/FS, 25.4 mV/dB
LCdB = sky[: ,channel]*cali # conversion into dB
LClin = 10.0**(LCdB/10.)

Tstart = 7.5 # start calibration > sun-rise
Tstop = 11.5 # stop calibration <  sun-set
dT = 0.25 # calibration period in hours
N = int((Tstop-Tstart)/dT) # number of calibration steps
print ('Number of calibrations steps ',N)
    

plt.figure()# present raw data from spectral verview
plt.plot(time,LClin,'-',color='green',label = header[channel] )
plt.xlabel("Time [UT]")
plt.ylabel("Intensity [linear]")
plt.title('Light curve raw data '+path)
#plt.xlim([Tstart,Tstop])
#plt.ylim(0,10)
plt.yscale('log')
plt.grid('both', which='minor', color='r', linestyle='-')
plt.grid('both', which='major', color='r', linestyle='-')
plt.legend(loc="upper left")
plt.savefig(path+'_'+header[channel]+'_LC.png',bbox_inches='tight')
plt.show()

#------------------------------------------------------------------------------

mask = (time > Tstart) * (time < Tstop) # limit analysis to the range of no obstructions

time = time[mask]
LClin = LClin[mask]
M = len(LClin) # number of measurement point within selected time-range

Acold = []
Ahot  = []
AtimC = []
AtimH = []
j = 0
i = 0
StartCold = 45.0 # seconds after calibration command. Find out by looking into details of light-curve
StopCold  = 75.0 # seconds after calibration command. Find out by looking into details of light-curve
while (i < M):
    t = time[i]
    y = LClin[i]
    T1 = Tstart + j*dT + StartCold / 3600 # 22 start cold sky
    T2 = Tstart + j*dT + StopCold  / 3600 # 51 stop cold sky
    Icold = 0
    k = 0
    if ((t >= T1) and (t <= T2)):
        Icold = Icold + y
        k = k + 1
        print ('i, t, y: ',i,t,y)

        Icold = Icold / k
        print ('Icold, k: ',Icold,k,'\n')
        Acold.append(Icold)
        AtimC.append(t)
        j = j + 1
    else:
        i = i + 1
    
j = 0
i = 0
StartHot = 108.0 # seconds after calibration command. Find out by looking into details of light-curve
StopHot  = 140.0 # seconds after calibration command. Find out by looking into details of light-curve
while (i < M):
    t = time[i]
    y = LClin[i]
    T1 = Tstart + j*dT + StartHot / 3600 # 82 start hot ground
    T2 = Tstart + j*dT + StopHot  / 3600 # 125 stop hot ground
    Ihot = 0
    k = 0
    if ((t >= T1) and (t <= T2)):
        Ihot = Ihot + y
        k = k + 1
        print ('i, t, y: ',i,t,y)

        Ihot = Ihot / k
        print ('Ihot, k: ',Ihot,k,'\n')
        Ahot.append(Ihot)
        AtimH.append(t)
        j = j + 1
    else:
        i = i + 1

#------------------------------------------------------------------------------

x = AtimC # calibration time for cold
y = Acold # amplitude for cold
order = 3 # polynom order to fit hot & cold (1...6)

ColdCoeff = np.polyfit(x, y, deg=order)
yC = np.polyval(ColdCoeff, x)

x = AtimH # calibration time for hot
y = Ahot  # amplitude for hot

HotCoeff = np.polyfit(x, y, deg=order)
yH = np.polyval(HotCoeff, x)


IIhot  = np.polyval(HotCoeff, time)
IIcold = np.polyval(ColdCoeff,time)

plt.figure()
plt.plot(time,IIcold,color='b',label='Cold sky')
plt.plot(time,IIhot, color='r',label='Hot ground')
plt.plot(time,LClin, color='black',label = 'Data')
plt.xlabel("Time [UT]")
plt.ylabel("Intensity [linear]")
plt.title('Hot and cold linear interpolation '+path)
plt.legend(loc="upper left")
plt.savefig(path+'_'+header[channel]+'_Interpolation.png',bbox_inches='tight')
plt.show()

#------------------------------------------------------------------------------

Thot  = 300.  # 273.15 K + Tamb
Tcold = 35.  # depends on publication taken into account 12 K ... 37 K
Tsun  = (Thot-Tcold)/(IIhot-IIcold) * (LClin-IIcold) + Tcold

plt.figure()
plt.plot(time,Tsun,color='purple')
plt.xlabel("Time [UT]")
plt.ylabel("Tsun [Kelvin]")
plt.title('Calibrated data '+path)
plt.savefig(path+'_'+header[channel]+'_Calibration.png',bbox_inches='tight')
plt.show()

#------------------------------------------------------------------------------

freqstring = header[channel] # selected frequency channel
frequency = float(freqstring[0:9]) # WFrequency in MHz
Lambda = 3e8/(frequency * 1e6) # Wavelength in meter
Bk = 1.38e-23 # Boltzmann constant in J/K
Gain = 10**3.5 # 35 dB antenna gain
sfu = 1e22 # conversion flux into sfu
Ssun = 8*np.pi*Bk*Tsun / (Lambda**2 * Gain) * sfu
correction = 0.87 # my observation / Learmont average interpolated via EXCEL
Ssun = Ssun * correction # based on cross-calibration
print ('Median flux: {:4.0f} sfu'.format(np.median(Ssun))) # observed flux
Squiet = 2.79e-5 * frequency**1.748 # quiet Sun flux after A. O. Benz
print ('Benz quiet flux: {:4.0f} sfu'.format(Squiet))

plt.figure()
plt.plot(time,Ssun,color='r')
plt.xlabel("Time [UT]")
plt.ylabel("Ssun [sfu]")
plt.title('Flux calibrated data '+path)
plt.savefig(path+'_'+header[channel]+'_SFU.png',bbox_inches='tight')
plt.show()

#------------------------------------------------------------------------------
