﻿# -*- coding: utf-8 -*-
"""
This script tracks the Sun in azimuthal mode (azimuth and elevation)
Carefully check paramter in MyLocation
It is sufficient to execute this script once every minute
Created on Thu Sep 14 20:13:50 2017
Updated: 09.05.2020, 05.05.2023, 13.05.2023
@author: Chr. Monstein
"""
# http://rhodesmill.org/pyephem/tutorial.html
# http://rhodesmill.org/pyephem/quick#other-observer-methods
#------------------------------------------------------------------------------

import ephem
import serial
import datetime
import math
import os.path

#------------------------------------------------------------------------------

MyLocation = ephem.Observer()
MyLocation.lon      = '8.755948' # east +°
MyLocation.lat      = '47.20329' # north +°
MyLocation.elev     = 414 # altitude in m asl
MyLocation.temp     = 20  # °C
MyLocation.pressure = 900 # mbar

MaxRange  = 77 # (+/- value) depends on your SAT-rotor type, check data sheet
AziMin    =  80.0 # value depends on local constrains
AziMax    = 245.0 # value depends on local constrains
EleMin    = -12.0 # value depends on local constrains
EleMax    = 90.0  # value depends on local constrains
AziPark   = 180.0 # value depends on local constrains
ElePark   = 80.0  # value depends on local constrains

AziCali   = 130.0 # cold sky reference for calibration
dAziCali  =  10.0 # +/- deviation from current azimuth
EleCali   =  45.0 # cold sky reference for calibration
EleGnd    = -10.0 # hot ground = ref ~300 kelvin
dEleCali  = -10.0 # +/- deviation from current elevation
TimeCali  =  15   # Time in modulo minute for cold sky calibration e.g. every 5, 10, 15, 20, 30 

# 0 = no calibration, 1 = fixposition azi,ele, 2 = deviate in azi, 3 = deviate in ele, 
# 4 = deviate in azi+ele, 5 = mode 3 and ground
CaliMode  = 5 # 5 mode of calibration

aziref = 171.0 # 163.0 deviation from south
eleref = 43.5 #  43.6, 45.0,46.5, 43.0, 45.5 (zenith+horizon)/2

azidir = -1.0 # rotation direction azimuth rotor, depends on mchanical mounting method
eledir =  1.0 # rotation direction elevation rotor, depends on mchanical mounting method

MyComport = 'COM13' # check with device manager (C:\Windows\System32\devmgmt.msc)

#------------------------------------------------------------------------------

def Park():
    RotorAzi = azidir*(AziPark - aziref) # conversion 0° ... 360° -> +/- 180°
    RotorEle = eledir*(ElePark - eleref)
    print ("Parking at Azimuth  = %7.2f  Elevation   = %6.2f" % (AziPark,ElePark))
    
    try:
        DiSEqC = serial.Serial(
             port     = MyComport,
             baudrate = 9600,
             bytesize = serial.EIGHTBITS,
             parity   = serial.PARITY_NONE,
             timeout  = 2)
             
        if (DiSEqC.isOpen()):
            print ("Successfully connected to antenna tracker at: "+DiSEqC.portstr)
            cmd = 'max{:6.2f}\r'.format(MaxRange) 
            DiSEqC.write(cmd.encode()) # set MaxRange
            
            cmd = 'azi{:6.2f}\r'.format(RotorAzi) 
            DiSEqC.write(cmd.encode()) # set azimuth or hour angle
            
            cmd = 'ele{:6.2f}\r'.format(RotorEle) 
            DiSEqC.write(cmd.encode()) # set elevation or declination
            
            DiSEqC.close()
        
    except IOError:
        print ("Problem communication with tracker!")
    
#------------------------------------------------------------------------------

def Log(x,y,msg):
    dat = datetime.datetime.now().strftime("%Y-%m-%d") # "%Y-%m-%d %H:%M:%S"
    filename ='DISEQ-'+dat+'.txt'
    if (os.path.exists(filename)):
        with open(filename, "a") as fp:   # Save x/y-data in file
            hms = datetime.datetime.now().strftime("%H:%M:%S") # "%Y-%m-%d %H:%M:%S"
            # add decimal time
            st = hms +',{:8.3f},'.format(x) + '{:8.3f}, '.format(y) + msg
            fp.write(st+'\n') 
    else:
        with open(filename, "a") as fp:   # Save x/y-data in file
            fp.write('DiSEqC radio telescope tracker\n') # write header information
            fp.write('Author: Christian Monstein, HB9SCT\n') # write header information
            fp.write('Version: 2023-04-27\n') # write header information
            fp.write('\n')
            hms = datetime.datetime.now().strftime("%H:%M:%S") # "%Y-%m-%d %H:%M:%S"
            fp.write('TIME [UT], Azimuth[°], Elevation[°], Message\n') # write header information
            st = hms +',{:8.3f},'.format(aziref) + '{:8.3f}, '.format(eleref) + 'Reference positions.\n'
            fp.write(st + '\n') 

#------------------------------------------------------------------------------

dt = datetime.datetime.now() # PC must run on UT or GPS-time
MyLocation.date = '{:4d}/{:02d}/{:02d} {:02d}:{:02d}:{:02d}'.format(
   dt.year,dt.month,dt.day,dt.hour,dt.minute,dt.second)
#MyLocation.date = '2023/03/14 07:24:00' # example for testing at high noon
print ('Selected date-time: ',MyLocation.date,' UT')

sun = ephem.Sun()
sun.compute(MyLocation)
azi = math.degrees(sun.az)
ele = math.degrees(sun.alt)
print("Sun data: Azimuth   = %7.2f  Elevation   = %6.2f" % (azi,ele))

lst = MyLocation.sidereal_time()
ha = (lst - sun.ra)/math.pi*180.0
dec = math.degrees(sun.dec)
print("Sun data: Hourangle = %7.2f  Declination = %6.2f" % (ha,dec))

if (((dt.minute%TimeCali) ==0) and (CaliMode ==1)): # calibration at fix position
    azi = AziCali
    ele = EleCali
    Log(azi,ele,'Cold sky reference at fix position')
    
if (((dt.minute%TimeCali) ==0) and (CaliMode ==2)): # calibration cold sky left or right
    azi = azi + dAziCali
    Log(azi,ele,'Cold sky reference deviation in azimuth')
    
if (((dt.minute%TimeCali) ==0) and ((CaliMode ==3) or (CaliMode ==5))): # calibration cold sky up or down
    ele = ele + dEleCali
    Log(azi,ele,'Cold sky reference deviation in elevation')
    
if (((dt.minute%TimeCali) ==0) and (CaliMode ==4)): # calibration both axis up or down and left or write
    azi = AziCali + dAziCali
    ele = EleCali + dEleCali
    Log(azi,ele,'Cold sky reference deviation in azimuth and elevation (diagonal)')
    
if ((((dt.minute-1)%TimeCali) ==0) and (CaliMode ==5)): # calibration hot ground low elevation
    ele = EleGnd
    Log(azi,ele,'Hot reference on ground (To)')

    
#------------------------------------------------------------------------------
# tracking block
if ((azi > AziMax) or (ele < EleMin) ):
    Park()
else:
    if (azi < AziMin):
        Log(azi,ele,'Azimuth out of east range')
        azi = AziMin
    else:
        if (azi > AziMax):
            Log(azi,ele,'Azimuth out of west range')
            azi = AziMax
    
    if (ele < EleMin):
        Log(azi,ele,'Elevation out of low range, too close to horizon')
        ele = EleMin
    else:
        if (ele > EleMax):
            Log(azi,ele,'Elevation out of high range, too close to zenith')
            ele = EleMax
            
    RotorAzi = azidir*(azi - aziref) # conversion 0° ... 360° -> +/- 180°
    RotorEle = eledir*(ele - eleref)
    
    print("Rotor positions: Azimuth   = %7.2f  Elevation   = %6.2f" % (RotorAzi,RotorEle))
    Log(azi,ele,'New tracking position')   
    
    try:
        DiSEqC = serial.Serial(
             port     = MyComport,
             baudrate = 9600,
             bytesize = serial.EIGHTBITS,
             parity   = serial.PARITY_NONE,
             timeout  = 2)
             
        if (DiSEqC.isOpen()):
            print ("Successfully connected to antenna tracker at: "+DiSEqC.portstr)
            cmd = 'max{:6.2f}\r'.format(MaxRange) 
            DiSEqC.write(cmd.encode()) # set MaxRange
            
            cmd = 'ele{:6.2f}\r'.format(RotorEle) 
            DiSEqC.write(cmd.encode()) # set elevation or declination
            
            cmd = 'azi{:6.2f}\r'.format(RotorAzi) 
            DiSEqC.write(cmd.encode()) # set azimuth or hour angle
            
            DiSEqC.close()
        
    except IOError:
        print ("Problem communication with elevation tracker!")

#------------------------------------------------------------------------------